package de.lmu.ifi.mfchords.utils;

/**
 * This class contains all hand-centric relative measurements. 
 * 
 * In the following 'b1' and 'b2' stands for the two touches of the multi-finger chord basis (thumb-pinky and thumb-index) 
 * or in the case of the neighboring fingers family for the two outer touches. 'p' stands for the third remaining touch.
 * 
 * 
 */
public class RelativeMeasurement {
	//in the case of three neighbouring fingers, the finger in the middle of the three is p.
		private Double D1; // between b1 and p
		private Double D2; // between p and b2
		private Double D3; // between b1 and b2
		
		private Double RD1;// = d1/d3; 
		private Double RD2;// = d1/d2;
		private Double RD3;// = d2/d3;
		
		
		private Double RA1; //angle at p1
		private Double RA2;//angle at p2
		private Double RA3;//angle at p0
		
		private Double RPD1; // relative distance of b1 to line between p and b2;
		private Double RPD2; // relative distance of p to line between b1 and b2;
		private Double RPD3; // relative distance of b2 to line between b1 and p;
		
		private Double RP1; // relative position of b1 on line between p and b2;
		private Double RP2; //relative position of p on line between b1 and b2;
		private Double RP3; //relative position of b2 on line between b1 and p;
		
		private String ID;
		
		
		private String entry;// this is saved to file
		
		/* Returns the identiy of the multi-finger chord (list of names are in Constants.java)  */
		public String getID(){
			return ID;
		}
		
		public RelativeMeasurement(String identifier){
			this.ID = identifier;
		}
		
		public RelativeMeasurement(){
			
		}
		
		
		protected RelativeMeasurement(double[] instance, String k){
			RD1 = instance[0];
			RD2 = instance[1];
			RD3 = instance[2];
			
			RA1 = instance[3];
			RA2 = instance[4];
			RA3 = instance[5];
			
			RPD1 = instance[6];
			RPD2 = instance[7];
			RPD3 = instance[8];
			
			RP1 = instance[9];
			RP2 = instance[10];
			RP3 = instance[11];
			
			ID = k;
		}
		
		protected double[] getInstanceMeasures(){
			double[] instanceM = new double[12];
			
			instanceM[0] = RD1;
			instanceM[1] = RD2;
			instanceM[2] = RD3;
			
			instanceM[3] = RA1;
			instanceM[4] = RA2;
			instanceM[5] = RA3;
			
			instanceM[6] = RPD1;
			instanceM[7] = RPD2;
			instanceM[8] = RPD3;
			
			instanceM[9] = RP1;
			instanceM[10] = RP2;
			instanceM[11] = RP3;
			
			return instanceM;
		}
		
		public void setID(String i){
			ID = i;
		}
		
		public String toString(){
			if(getEntry()!= null && !getEntry().isEmpty()){
				return getEntry();
			}
			String returnString = "********************* \n relative measurement: " + ID +  "\n";
			returnString = returnString.concat("RD1:" + RD1 + " RD2:" + RD2 + " RD3:" + RD3);
			returnString = returnString.concat("RA1:" + RA1 + " RA2:" + RA2 + " RA3:" + RA3);
			returnString = returnString.concat("RPD1:" + RPD1 + " RPD2:" + RPD2 + " RPD3:" + RPD3);
			returnString = returnString.concat("RP1:" + RP1 + " RP2:" + RP2 + " RP3:" + RP3);
			return returnString;
		}

		/* Distance between Basis touch point b1 and the identifying touch point p*/
		public Double getD1() {
			return D1;
		}

		
		public void setD1(Double d1) {
			D1 = d1;
		}

		/** Distance between identifying touch point p and basis touch point b2*/
		public Double getD2() {
			return D2;
		}

		public void setD2(Double d2) {
			D2 = d2;
		}

		/** Distance between basis touch points b1 and b2*/
		public Double getD3() {
			return D3;
		}

		public void setD3(Double d3) {
			D3 = d3;
		}

		/** Relative distance between b1 and p */
		public Double getRD1() {
			return RD1;
		}

		public void setRD1(Double rD1) {
			RD1 = rD1;
		}

		/** Relative distance between p and b2 */
		public Double getRD2() {
			return RD2;
		}

		public void setRD2(Double rD2) {
			RD2 = rD2;
		}

		
		/* Relative distance between b1 and b2 */
		public Double getRD3() {
			return RD3;
		}

		public void setRD3(Double rD3) {
			RD3 = rD3;
		}

		
		/** Relative angle between b1 and p */
		public Double getRA1() {
			return RA1;
		}

		public void setRA1(Double rA1) {
			RA1 = rA1;
		}

		/** Relative angle between p and b2 */
		public Double getRA2() {
			return RA2;
		}

		public void setRA2(Double rA2) {
			RA2 = rA2;
		}

		/** Relative angle between b2 and b1 */
		public Double getRA3() {
			return RA3;
		}

		public void setRA3(Double rA3) {
			RA3 = rA3;
		}

		
		/** Relative projection distance: the relative distance of b2 projected onto b1-p to the distance of b1-p*/
		public Double getRPD1() {
			return RPD1;
		}

		public void setRPD1(Double rPD1) {
			RPD1 = rPD1;
		}

		/** Relative projection distance: the relative distance of b1 projected onto p-b2 to the distance of p-b2*/
		public Double getRPD2() {
			return RPD2;
		}

		public void setRPD2(Double rPD2) {
			RPD2 = rPD2;
		}

		/** Relative projection distance: the relative distance of p projected onto b1-b2 to the distance of b1-b2*/
		public Double getRPD3() {
			return RPD3;
		}

		public void setRPD3(Double rPD3) {
			RPD3 = rPD3;
		}

		/** Relative projection: the relative position of b2 projected onto b1-p*/
		public Double getRP1() {
			return RP1;
		}

		public void setRP1(Double rP1) {
			RP1 = rP1;
		}
		
		/** Relative projection: the relative position of b1 projected onto p-b2*/
		public Double getRP2() {
			return RP2;
		}

		public void setRP2(Double rP2) {
			RP2 = rP2;
		}

		/** Relative projection: the relative position of p projected onto b1-b2*/
		public Double getRP3() {
			return RP3;
		}

		
		public void setRP3(Double rP3) {
			RP3 = rP3;
		}
		
		
		

		
		public String getEntry() {
			return ""+ RD1 + ";" + RD2 + ";" + RD3 + ";" + 
						RA1 + ";" + RA2 + ";" + RA3 + ";" +
						RPD1 + ";" + RPD2 + ";" + RPD3 + ";" +
						RP1 + ";" + RP2 + ";" + RP3 + ";" + ID + "\n";
		}

		public void setEntry(String entry) {
			this.entry = entry;
		}
	
}
